MODULE ManyWind;

(*
	This application allows up to twenty windows to be created at once, with the
	New item under the File menu.  The name of each window appears under the
	Windows menu (which is not created until at least one window exists).
	Selecting the window name from the Windows menu brings the window to the
	front.  For every window created, Skel is told to create a new handler.  If
	the window's close box is clicked, the handler removes the window name from
	the Windows menu, disposes of the window, and removes itself from the window
	handler list.  If the window was the last window, the Windows menu handler
	removes itself from the menu handler list.
	
	When the first window is created, a Color menu also appears.  This allows the
	pattern of the content region of the frontmost window to be changed.  It goes
	away when the last window is closed.
	
	To quit, select Quit from the File menu or type Command-Q.
	
	ManyWind demonstrates dynamic window and menu creation and disposal.  It
	also shows how handler procedures may be shared among handlers for different
	windows.
	
	 Paul DuBois	 28 June 1986
		TML Modula-2 version by Dennis Cohen  16 April 1987
*)

FROM SYSTEM IMPORT ADDRESS, ADR;
FROM QuickDraw IMPORT Rect, GrafPtr, GetPort, EraseRect, OffsetRect, BackPat,
											SetRect, white, ltGray, gray, dkGray, black;
FROM MenuManager IMPORT MenuHandle, NewMenu, AppendMenu, DisableItem, EnableItem,
												GetItem, CountMItems, DisposeMenu;
FROM WindowManager IMPORT WindowPtr, NewWindow, documentProc, GetWTitle,
													DisposeWindow, SelectWindow, WindowPeek, FrontWindow;
FROM Strings IMPORT Str255;
FROM Utilities IMPORT EqualString;
FROM BDPackage IMPORT NumToString;
FROM TransSkel IMPORT SkelInit, SkelApple, SkelGrowBounds, SkelMenu, SkelMain,
											SkelClobber, SkelWhoa, SkelWindow, WMouseProc, WKeyProc,
											WActivateProc, WIdleProc, SkelRmveWind, SkelRmveMenu;

CONST
	maxWind = 20;

	aMenuNum = 1;	(* Apple Menu *)
	fMenuNum = 2;	(* File Menu *)
		new = 1;
		quit = 3;
	wMenuNum = 3;	(* Windows Menu *)
	cMenuNum = 4;	(* Color Menu *)
		cWhite = 1;
		cLtGray = 2;
		cGray = 3;
		cDkGray = 4;
		cBlack = 5;

VAR
	fileMenu,
	windowMenu,
	colorMenu:	MenuHandle;
	
	windCount: INTEGER;
	windNum: LONGINT;
	dummy: LONGINT;

PROCEDURE DoWUpdate(resized: BOOLEAN);	(* Ignores Grow *)

	VAR
		thePort: GrafPtr;

BEGIN
	GetPort(thePort);
	EraseRect(thePort^.portRect);	(* repaint with background pattern *)
END DoWUpdate;


PROCEDURE DoWClobber;
(*
 *	Dispose of window.  Skel makes sure the port is pointing to the appropriate
 *	window, so this procedure can determine which window is to be disposed of,
 *	without being told explicitly.
 *)
	VAR
		thePort: GrafPtr;

BEGIN
	GetPort(thePort);
	DisposeWindow(thePort);
END DoWClobber;


PROCEDURE DoWindowMenu(item: INTEGER);

	VAR
		iTitle, wTitle: Str255;
		w: WindowPeek;

BEGIN
	GetItem(windowMenu, item, iTitle);
	w := VAL(WindowPeek, FrontWindow());
	LOOP
		IF w = NIL THEN EXIT; END;
		GetWTitle(VAL(WindowPtr, w), wTitle);
		IF EqualString(iTitle, wTitle, FALSE, TRUE) THEN
			SelectWindow(VAL(WindowPtr, w));
			EXIT;
		END;
		w := w^.nextWindow;
	END;
END DoWindowMenu;


PROCEDURE DoColorMenu(item: INTEGER);
(*
 *	Change the background pattern of the frontmost window.  Ignore if it's a DA.
 *)
	VAR
		w: WindowPeek;

BEGIN
	w := VAL(WindowPeek, FrontWindow());
	IF w^.windowKind < 0 THEN RETURN; END; (* It belongs to the system *)
	CASE item OF
		cWhite: BackPat(white);
		|
		cLtGray: BackPat(ltGray);
		|
		cGray: BackPat(gray);
		|
		cDkGray: BackPat(dkGray);
		|
		cBlack: BackPat(black);
		ELSE (* do nothing *)
	END;
	EraseRect(w^.port.portRect);
END DoColorMenu;


PROCEDURE DoMClobber(theMenu: MenuHandle);

BEGIN
	DisposeMenu(theMenu);
END DoMClobber;


PROCEDURE DoWClose;

	VAR
		thePort: GrafPtr;
		m: MenuHandle;
		i, mItems: INTEGER;
		iTitle, wTitle: Str255;

BEGIN
	GetPort(thePort);
	GetWTitle(thePort, wTitle);
	SkelRmveWind(thePort);
	DEC(windCount);
	IF windCount = 0 THEN	(* last window, clobber the menus *)
		SkelRmveMenu(windowMenu);
		SkelRmveMenu(colorMenu);
	ELSE
(*
 *		just take out of menu  this could be done better with DelMenuItem,
 *		but the original didn't use it and then it wouldn't work on 64K ROMS
 *)
		m := NewMenu(wMenuNum, "Windows");
		mItems := CountMItems(windowMenu);
		FOR i := 1 TO mItems DO
			GetItem(windowMenu, i, iTitle);
			IF NOT EqualString(iTitle, wTitle, FALSE, TRUE) THEN
				AppendMenu(m, iTitle);
			END;
		END;
		SkelRmveMenu(windowMenu);
		windowMenu := m;
		SkelMenu(windowMenu, DoWindowMenu, DoMClobber);
	END;
	EnableItem(fileMenu, new);	(* can always create at least one more now *)
END DoWClose;


PROCEDURE MakeWindow;
(*
	Make new window.  Locate at (100, 100) if no other window exists, else offset
	slightly from front window.  The window title is the next window number.  If
	this is the first window, create the Windows and Color menus.  Add the window
	title as the last item of the Windows menu.
	
	If the maximum window count has been reached, disable new in the File menu.
*)
	VAR
		w: WindowPtr;
		r, r2: Rect;
		s: Str255;

BEGIN
	SetRect(r, 0, 0, 200, 150);
	w := FrontWindow();
	IF w = NIL THEN OffsetRect(r, 100, 100);
	ELSE
		r2 := w^.portBits.bounds;
		OffsetRect(r, 20-r2.left, 20-r2.top);
		IF (r.left > 480) OR (r.top > 300) THEN
			OffsetRect(r, 40-r.left, 40-r.top);
		END;
	END;
	INC(windNum);
	NumToString(windNum, s);
	w := NewWindow(NIL, r, s, TRUE, documentProc, VAL(WindowPtr, -1D), TRUE, 0D);
	SkelWindow(	w,
							VAL(WMouseProc, NIL),
							VAL(WKeyProc, NIL),
							DoWUpdate,
							VAL(WActivateProc, NIL),
							DoWClose,
							DoWClobber,
							VAL(WIdleProc, NIL),
							FALSE);
	IF windCount = 0 THEN
		INC(windCount);
		colorMenu := NewMenu(cMenuNum, "Color");
		AppendMenu(colorMenu, "White;Light Gray;Gray;Dark Gray;Black");
		SkelMenu(colorMenu, DoColorMenu, DoMClobber);
		windowMenu := NewMenu(wMenuNum, "Windows");
		SkelMenu(windowMenu, DoWindowMenu, DoMClobber);
	ELSE INC(windCount);
	END;
	AppendMenu(windowMenu, s);
	IF windCount = maxWind THEN DisableItem(fileMenu, new); END;
END MakeWindow;


PROCEDURE DoFileMenu(item: INTEGER);

BEGIN
	IF item = new THEN MakeWindow;			(* Make a new window *)
	ELSIF item = quit THEN SkelWhoa;	(* Tell TransSkel.SkelMain to quit *)
	END;
END DoFileMenu;


BEGIN
	windCount := 0;
	windNum := 0D;
	SkelInit;
	SkelApple("", VAL(PROC, NIL));
	fileMenu := NewMenu(fMenuNum, "File");
	AppendMenu(fileMenu, "New/N;(-;Quit/Q");
	SkelGrowBounds(NIL, 50, 10, 500, 300);
	SkelMenu(fileMenu, DoFileMenu, DoMClobber);
	SkelMain;
	SkelClobber;
END ManyWind.
